use strict;
use warnings FATAL => 'all';

=head1 NAME

Script::Build - extend Module::Build with functionality for scripts

=head1 SYNOPSIS

 my $build = new Script::Build(
     # ...usual Module:Build parameters here...
     pl_files => ['myscript.pl'],
 );
 $build->create_build_script;

=head1 DESCRIPTION

This extension to Module::Build adds support for "compiling" perl scripts,
that is, prepending the shebang header and (more importantly) some
information about installation paths.

The list of scripts to "compile" is taken from the C<pl_files> parameter as
shown above, which can be an array reference or a hash reference, just like
the normal Module::Build C<script_files> parameter. If no C<pl_files>
parameter is given, Script::Build will search for C<*.pl> files in the
F<script/> subdirectory.

Each C<.pl> file thus specified will be copied verbatim to a file with the
same name but without the C<.pl> extension and with an installation
dependant blurb prepended to it. This header will contain the shebang line
(typically C<#! /usr/bin/perl>) and a hash called C<%DIR> containing
installation paths for various components. Such a blurb will typically look
something like:

 #! /usr/bin/perl

 my %DIR = (
     arch => '/usr/lib/perl5',
     bin => '/usr/bin',
     bindoc => '/usr/share/man/man1',
     binhtml => undef,
     conf => '/etc',
     lib => '/usr/share/perl5',
     libdoc => '/usr/share/man/man3',
     libhtml => undef,
     root => '/usr',
     script => '/usr/bin',
 );

Which would be followed by your script, possibly looking something like:

 use strict;
 use warnings FATAL => 'all';

 open CONFIG, '<', "$DIR{conf}/frobber.conf" or die;

The elements of the C<%DIR> hash are determined by Module::Build but with
the addition of the C<conf> entry. This entry can be manipulated in the
same way as other paths in Module::Build.

=cut

package Script::Build;

use base qw(Module::Build);

use IO::File;
use File::Spec::Functions qw(catdir);

*file_qr = \&Module::Build::Base::file_qr;

sub new {
	my $class = shift;
	my $self = $class->SUPER::new(@_);
	$self->add_build_element('pl');
	return $self;
}

sub installation_prefix {
	my $self = shift;
	return $self->install_base if defined $self->install_base;
	return $self->prefix if defined $self->prefix;
	return $self->original_prefix->{$self->installdirs};
}

sub _set_install_paths {
	my $self = shift;

	my $ret = $self->SUPER::_set_install_paths(@_);

	my $props = $self->{properties};
	my $config = $self->{config};

	my $install_sets = $props->{install_sets};
	while(my ($installtype, $installdirs) = each(%$install_sets)) {
		my $infix = $installtype eq 'core' ? '' : $installtype;
		my $prefix = $self->original_prefix->{$installtype};
		my $confdir = catdir($prefix eq '/usr' ? '/' : $prefix, 'etc')
			if defined $prefix;
		$installdirs->{conf} = $config->get("install${infix}conf") || $confdir
			unless exists $installdirs->{conf};
	}
	unless(exists $props->{install_base_relpaths}{conf}) {
		$props->{install_base_relpaths}{conf} = ['etc']
	}
	my $prefix_relpaths = $props->{prefix_relpaths};
	while(my ($installtype, $installdirs) = each(%$prefix_relpaths)) {
		$installdirs->{conf} = ['etc']
			unless exists $installdirs->{conf};
	}
	
	return $ret;
}

sub process_pl_files {
	my $self = shift;
	my $files = $self->find_pl_files;

	my $perl = $self->perl;

	my $script = catdir($self->blib, 'script');
	my $tmpdir = catdir($self->blib, 'tmp');
	File::Path::mkpath($script);
	File::Path::mkpath($tmpdir);

	my $installdirs = $self->installdirs;
	my %paths;
	@paths{keys %{$self->install_sets->{$installdirs}}} = ();
	@paths{keys %{$self->prefix_relpaths->{$installdirs}}} = ();
	@paths{keys %{$self->install_base_relpaths}} = ();
	foreach my $path (keys %paths) {
		$paths{$path} = $self->install_destination($path);
	}
	$paths{root} = $self->installation_prefix();

	while (my ($src, $dsts) = each %$files) {
		my @out;
		my @names;
		foreach my $dst (@$dsts) {
			my $base = File::Basename::basename($dst);
			my $to = File::Spec->catfile($script, $base);
			next if $self->up_to_date($src, $to);

			print STDERR "Building $src -> $to\n";
			push @names, $base;
			my $tmp = File::Spec->catfile($tmpdir, $base);
			my $fh = new IO::File($tmp, O_WRONLY|O_CREAT|O_TRUNC, 0777)
				or die "$to: $!\n";
			push @out, $fh;
			$fh->binmode;
			$fh->print("#! $perl\n\nmy \%DIR = (\n")
				or die "$to: $!\n";
			foreach my $type (sort keys %paths) {
				my $val = $paths{$type};
				$val = defined $val ? "'$val'" : 'undef';
				$type = "'$type'"
					unless $type =~ /^[a-z_][a-z0-9_]*$/i;
				$fh->print("\t$type => $val,\n")
					or die "$to: $!\n";
			}
			$fh->print(");\n\n")
				or die "$to: $!\n";
		}
		next unless @out;
		my $in = new IO::File($src, O_RDONLY)
			or die "$src: $!\n";
		$in->binmode;
		while(<$in>) {
			foreach my $fh (@out) {
				$fh->print($_)
					or die "can't write: $!\n";
			}
		}
		$in->close;
		foreach my $fh (@out) {
			$fh->close
				or die "can't close: $!\n";
		}
		foreach my $base (@names) {
			my $to = File::Spec->catfile($script, $base);
			my $tmp = File::Spec->catfile($tmpdir, $base);
			rename($tmp, $to)
				or die "rename($tmp, $to): $!\n";
		}
	}
}

sub find_pl_files {
	my $self = shift;
	my $files = $self->{properties}{pl_files};
	if(UNIVERSAL::isa($files, 'ARRAY')) {
		return {
			map {$_, [/^(.*)\.pl$/]}
			map $self->localize_file_path($_),
			@$files
		};
	} elsif(UNIVERSAL::isa($files, 'HASH')) {
		my %out;
		while(my ($file, $to) = each %$files) {
			$out{$self->localize_file_path($file)} = [
				map $self->localize_file_path($_), ref $to ? @$to : ($to)
			];
		}
		return \%out;
	} elsif(defined $files) {
		die "'pl_files' must be a hash reference or array reference";
	}

	return unless -d 'script';
	return {
		map {$_, [/^(.*)\.pl$/i]} @{
			$self->rscan_dir('script', file_qr('\.pl$'))
		}
	};
}

1;

=head1 AUTHOR

Wessel Dankers <wsl@fruit.je>

=head1 COPYRIGHT

Copyright (c) 2009 Wessel Dankers <wsl@fruit.je>

This library is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=head1 SEE ALSO

Module::Build(3)
